/*
 * Copyright (c) 2015, AIOI・SYSTEMS CO., LTD.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */
package com.aioisystems.marbleportsample;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.FontMetrics;
import android.nfc.Tag;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

public class MarblePortSampleActivity extends Activity {

    private static final int ITEM_SHOW_STATUS = 0;
    private static final int ITEM_GET_STATUS = 1;
    private static final int ITEM_SHOW_IMAGE = 2;
    private static final int ITEM_SHOW_TEXT = 3;
    private static final int ITEM_WRITE_DATA = 4;
    private static final int ITEM_READ_DATA = 5;
    private static final int ITEM_REGISTER_IMAGE = 6;
    private static final int ITEM_SHOW_REGISTERED_IMAGE = 7;
    private static final int ITEM_CLEAR_DISPLAY = 8;
    private static final int ITEM_COMBINATION_1 = 9;
    private static final int ITEM_TAG_DETECTION = 10;
    private static final int ITEM_SHOW_PARTIAL_IMAGE = 11;
    private static final int ITEM_MULTI_FUNCTION = 12;

    private static final int ITEM_SHOW_IMAGE27 = 13;
    private static final int ITEM_REGISTER_IMAGE27 = 14;
    private static final int ITEM_CHANGE_SECURITY_CODE = 15;

    private static final int ITEM_SHOW_IMAGE29 = 16;

    private static final int ITEM_COMBINATION_2 = 101;

    private Bitmap mBitmap;
    private Bitmap mBitmap27;
    private Bitmap mBitmap29;
    private TextView mMsgView;
    private TextView mOutputView;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        checkMarblePort();

        //load image resource.
    	mBitmap = BitmapFactory.decodeResource(
    					getResources(),
    					R.drawable.sampleimage);

    	mBitmap27 = BitmapFactory.decodeResource(
    					getResources(),
    					R.drawable.sampleimage2);

    	mBitmap29 = BitmapFactory.decodeResource(
    					getResources(),
						R.drawable.sampleimage3);
        initUI();
    }

    @Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		mOutputView.setText("");
		if(resultCode == RESULT_CANCELED){
			mMsgView.setText(getString(R.string.msgCanceled));

		}else if(resultCode == MpHelper.RESULT_ERROR_CANCEL){
			mMsgView.setText(getString(R.string.msgErrorCanceled));

		}else if(resultCode == RESULT_OK){
			mMsgView.setText(getString(R.string.msgDone2));

			Bundle bundle = data.getExtras();
			switch(requestCode){
			case ITEM_SHOW_STATUS:
				break;
			case ITEM_GET_STATUS:
				showGetStatusResult(bundle);
				break;
			case ITEM_READ_DATA:
				showReadData(bundle);
				break;
			case ITEM_COMBINATION_1:
				execCombination2(bundle);
				break;
			case ITEM_CHANGE_SECURITY_CODE:
				checkSecurityChangeResult(bundle);
				break;
			case ITEM_REGISTER_IMAGE27:
				showGetStatusResult(bundle);
				break;
			case ITEM_MULTI_FUNCTION:
				showReadData(bundle);
				break;
			}
		}
	}

    /**
     * Initialize u/i
     */
    private void initUI(){
    	mMsgView = (TextView)findViewById(R.id.msgView);
    	mOutputView = (TextView)findViewById(R.id.outputView);

    	mMsgView.setText("");
    	mOutputView.setText("");

    	//setup list view
    	ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
    	adapter.add(getString(R.string.showStatus));
    	adapter.add(getString(R.string.getStatus));
    	adapter.add(getString(R.string.showImage));
    	adapter.add(getString(R.string.showText));
    	adapter.add(getString(R.string.writeUserData));
    	adapter.add(getString(R.string.readUserData));
    	adapter.add(getString(R.string.registerImage));
    	adapter.add(getString(R.string.showRegisterdImage));
    	adapter.add(getString(R.string.clearDisplay));
    	adapter.add(getString(R.string.combination));
    	adapter.add(getString(R.string.tagDetection));
    	adapter.add(getString(R.string.showParialImage));
    	adapter.add(getString(R.string.multiFunction));
    	adapter.add(getString(R.string.showImage27));
    	adapter.add(getString(R.string.registerImage27));
    	adapter.add(getString(R.string.changeSecurityCode));
    	adapter.add(getString(R.string.showImage29));

    	ListView listView1 = (ListView)findViewById(R.id.listView1);
    	listView1.setAdapter(adapter);
    	listView1.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int arg2,
					long arg3) {

				try{
					switch(arg2){
					case ITEM_SHOW_STATUS:
						showStatus();
						break;
					case ITEM_GET_STATUS:
						getStatus();
						break;
					case ITEM_SHOW_IMAGE:
						showImage(mBitmap);
						break;

					case ITEM_SHOW_TEXT:
						showText();
						break;
					case ITEM_WRITE_DATA:
						writeUserData();
						break;
					case ITEM_READ_DATA:
						readUserData();
						break;
					case ITEM_REGISTER_IMAGE:
						registerImage();
						break;
					case ITEM_SHOW_REGISTERED_IMAGE:
						showRegisteredImage();
						break;
					case ITEM_CLEAR_DISPLAY:
						clearDisplay();
						break;
					case ITEM_COMBINATION_1:
						execCombination1();
						break;
					case ITEM_TAG_DETECTION:
						Intent intent = new Intent(MarblePortSampleActivity.this, RelayActivity.class);
						startActivity(intent);
						break;
					case ITEM_SHOW_PARTIAL_IMAGE:
						showPartialImage();
						break;
					case ITEM_MULTI_FUNCTION:
						executeMultiFunction();
						break;
					case ITEM_SHOW_IMAGE27:
						showImage27(mBitmap27);
						break;
					case ITEM_REGISTER_IMAGE27:
						registerImage27(mBitmap27);
						break;
					case ITEM_CHANGE_SECURITY_CODE:
						changeSecurityCode();
						break;
					case ITEM_SHOW_IMAGE29:
						showImage29(mBitmap29);
						break;

					}
				}catch(ActivityNotFoundException e){
					checkMarblePort();
				}
			}
		});
    }

    /**
     * Make sure that is installed marblePORT.
     */
    private void checkMarblePort(){
    	if(!MpHelper.checkMarblePort(this)){
        	AlertDialog.Builder builder = new AlertDialog.Builder(this);

        	builder.setMessage(getString(R.string.msgGetMarblePort));
        	builder.setPositiveButton(getString(R.string.yes), new DialogInterface.OnClickListener() {

				@Override
				public void onClick(DialogInterface dialog, int which) {
					MpHelper.navigateToMarket(
							MarblePortSampleActivity.this);
				}
			});
        	builder.setNegativeButton(getString(R.string.No), null);
        	builder.setCancelable(true);
        	AlertDialog dialog = builder.create();
        	dialog.show();
        }
    }

    /**
     * Show the tag information to marblePORT screen.
     */
    private void showStatus(){
    	Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
    	//Required
    	intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_STATUS);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_AUTO_CLOSE, MpHelper.CLOSE_TYPE_NONE);

    	startActivityForResult(intent, ITEM_SHOW_STATUS);
    }

    /**
	 * Gets the tag information.
	 */
	private void getStatus(){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_GET_STATUS);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_AUTO_CLOSE, MpHelper.CLOSE_TYPE_IMMEDIATE);
    	startActivityForResult(intent, ITEM_GET_STATUS);
	}

	/**
	 * Display the tag information that was retrieved.
	 * @param bundle
	 */
	private void showGetStatusResult(Bundle bundle){
		byte[] idm = (byte[])bundle.getByteArray(MpHelper.EXTRA_IDM);
		byte battery = (byte)bundle.getByte(MpHelper.EXTRA_BATTERY, (byte)-1);
		byte status = (byte)bundle.getByte(MpHelper.EXTRA_TAG_STATUS, (byte)0);
		byte version = (byte)bundle.getByte(MpHelper.EXTRA_FIRMWARE_VERSION, (byte)0);

		String output = String.format(
				"IDm=%s\n"
				+ "Battery=0x%02X\n"
				+ "Status=0x%02X\n"
				+ "Version=0x%02X",
				makeHexText(idm),
				battery,
				status,
				version);
		mOutputView.setText(output);
	}

	/**
	 * Show an image to the smart tag display.
	 */
	private void showImage(Bitmap bitmap){

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		//Required
    	intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_IMAGE);
    	intent.putExtra(MpHelper.EXTRA_BITMAP, bitmap);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_DITHER, true);
    	intent.putExtra(MpHelper.EXTRA_AUTO_CLOSE, MpHelper.CLOSE_TYPE_DELAY);
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
    	intent.putExtra(MpHelper.EXTRA_VIBRATE, MpHelper.VIBRATE_ON);

    	//customizing display message
    	intent.putExtra(MpHelper.EXTRA_TITLE, getString(R.string.title));
    	intent.putExtra(MpHelper.EXTRA_MSG_SCAN, getString(R.string.msgScan));
    	intent.putExtra(MpHelper.EXTRA_MSG_PROCESSING, getString(R.string.msgProcessing));
    	intent.putExtra(MpHelper.EXTRA_MSG_END, getString(R.string.msgDone));
    	intent.putExtra(MpHelper.EXTRA_MSG_IO_ERROR, getString(R.string.msgError));

    	startActivityForResult(intent, ITEM_SHOW_IMAGE);
	}

	/**
	 * Show an image to the 2.7inch smart tag display.
	 */
	private void showImage27(Bitmap bitmap){

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		//Required
		intent.putExtra(MpHelper.EXTRA_SMART_TAG_TYPE, MpHelper.SMART_TAG_TYPE_27);
    	intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_IMAGE);
    	intent.putExtra(MpHelper.EXTRA_BITMAP, bitmap);

    	//Optional
    	intent.putExtra(MpHelper.EXTRA_DITHER, true);
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
    	intent.putExtra(MpHelper.EXTRA_SECURITY_CODE1, new byte[]{0x30, 0x30, 0x30});

    	startActivityForResult(intent, ITEM_SHOW_IMAGE);
	}

	/**
	 * Show an image to the 2.9inch SmartCard display.
	 */
	private void showImage29(Bitmap bitmap){

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		//Required
		intent.putExtra(MpHelper.EXTRA_SMART_TAG_TYPE, MpHelper.SMART_TAG_TYPE_29);
    	intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_IMAGE);
    	intent.putExtra(MpHelper.EXTRA_BITMAP, bitmap);

    	//Optional
    	intent.putExtra(MpHelper.EXTRA_DITHER, true);
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
    	intent.putExtra(MpHelper.EXTRA_SECURITY_CODE1, new byte[]{0x30, 0x30, 0x30});
    	intent.putExtra(MpHelper.EXTRA_CHECK_STATUS_AFTER_PROCESS, true);

    	startActivityForResult(intent, ITEM_SHOW_IMAGE);
	}

	/**
	 * Show partial image with the registered image.
	 */
	private void showPartialImage(){
		//Create the image.
		final int width = 50;
		final int height = 20;
		Bitmap bmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
		Canvas canvas = new Canvas(bmp);
		canvas.drawColor(Color.WHITE);

		//border
		Paint paint = new Paint();
    	paint.setColor(Color.BLACK);
    	paint.setStyle(Paint.Style.STROKE);
    	paint.setStrokeWidth(1);
    	canvas.drawRect(0, 0, width - 1, height - 1, paint);

    	//text
    	paint.reset();
    	paint.setAntiAlias(false);
    	paint.setColor(Color.BLACK);
    	paint.setTextSize(16);
    	FontMetrics metrics = paint.getFontMetrics();
    	float top = -1 * metrics.top;
    	canvas.drawText("Hello!", 1, top, paint);

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		//Required
    	intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_IMAGE);
    	intent.putExtra(MpHelper.EXTRA_BITMAP, bmp);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_LOCATION_X, 20);
    	intent.putExtra(MpHelper.EXTRA_LOCATION_Y, 30);
    	intent.putExtra(MpHelper.EXTRA_DRAW_MODE, MpHelper.DRAW_MODE_LAYOUT);
    	intent.putExtra(MpHelper.EXTRA_LAYOUT_NUMBER, 1);

    	startActivityForResult(intent, ITEM_SHOW_IMAGE);
	}

	/**
	 * Show a text to the smart tag display.
	 */
	private void showText(){
		String text;
		text = "Smart Tag or “Visual RFID Tag” display uses electronic paper. It can keep data displayed on its screen without using any electric power. Power is required when data on the display screen is rewritten or data is read from the tag. Using Smart Tags instead of paper labels saves time and reduces environmental problems.";
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_TEXT);
    	intent.putExtra(MpHelper.EXTRA_DISPLAY_TEXT, text);
    	intent.putExtra(MpHelper.EXTRA_TEXT_SIZE, 12);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);

    	startActivityForResult(intent, ITEM_SHOW_TEXT);
	}

	/**
	 * Write user data to the smart tag.
	 */
	private void writeUserData(){

		int testSize = 3072;
		byte[] data = new byte[testSize];
		for(int i = 0; i < testSize; i++){
			data[i] = (byte)(0x30 + (i % 10));
		}

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_WRITE_DATA);
    	intent.putExtra(MpHelper.EXTRA_START_ADDRESS, 0);
    	intent.putExtra(MpHelper.EXTRA_USER_DATA, data);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);

    	startActivityForResult(intent, ITEM_WRITE_DATA);
	}

	/**
	 * Read user data from the smart tag.
	 */
	private void readUserData(){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_READ_DATA);
    	intent.putExtra(MpHelper.EXTRA_START_ADDRESS, 0);
    	intent.putExtra(MpHelper.EXTRA_READ_SIZE, 3072);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
    	intent.putExtra(MpHelper.EXTRA_AUTO_CLOSE, MpHelper.CLOSE_TYPE_IMMEDIATE);

    	startActivityForResult(intent, ITEM_READ_DATA);
	}

	/**
	 * Show data read from the smart tag.
	 * @param bundle
	 */
	private void showReadData(Bundle bundle){

		byte[] userData = (byte[])bundle.getByteArray(MpHelper.EXTRA_USER_DATA);

		String text = "";
		try{
			text = new String(userData, "ASCII");
		}catch(Exception ignore){
		}

		mOutputView.setText(text);
	}

	/**
	 * Register a image as the layout 1.
	 */
	private void registerImage(){

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SAVE_LAYOUT);
    	intent.putExtra(MpHelper.EXTRA_LAYOUT_NUMBER, 1);

    	//Optional
    	intent.putExtra(MpHelper.EXTRA_BITMAP, mBitmap); //If nothing specified, registers the last shown image.
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);

    	startActivityForResult(intent, ITEM_REGISTER_IMAGE);
	}

	/**
	 * Registers a 2.7inch image as the layout 1.
	 */
	private void registerImage27(Bitmap bitmap){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_SMART_TAG_TYPE, MpHelper.SMART_TAG_TYPE_27);
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SAVE_LAYOUT);
    	intent.putExtra(MpHelper.EXTRA_LAYOUT_NUMBER, 1);

    	//Optional
    	intent.putExtra(MpHelper.EXTRA_BITMAP, bitmap);	//If nothing specified, registers the last shown image.
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
    	intent.putExtra(MpHelper.EXTRA_UPDATE_DISPLAY, true); //default:true
    	intent.putExtra(MpHelper.EXTRA_CHECK_STATUS_AFTER_PROCESS, true);

    	startActivityForResult(intent, ITEM_REGISTER_IMAGE27);
	}


	/**
	 * Show the image registered in the smart tag.
	 */
	private void showRegisteredImage(){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_CHANGE_LAYOUT);
    	intent.putExtra(MpHelper.EXTRA_LAYOUT_NUMBER, 1);

    	startActivityForResult(intent, ITEM_SHOW_REGISTERED_IMAGE);
	}

	/**
	 * Clear the smart tag dispaly.
	 */
	private void clearDisplay(){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_CLEAR_DISPLAY);
    	startActivityForResult(intent, ITEM_CLEAR_DISPLAY);

	}

	/**
	 * Perform a combination of multiple processing.
	 * 1/2: Read the count value.
	 */
	private void execCombination1(){
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_READ_DATA);
    	intent.putExtra(MpHelper.EXTRA_START_ADDRESS, 0);
    	intent.putExtra(MpHelper.EXTRA_READ_SIZE, 1);
    	//Optional
    	intent.putExtra(MpHelper.EXTRA_AUTO_CLOSE, MpHelper.CLOSE_TYPE_IMMEDIATE);
    	intent.putExtra(MpHelper.EXTRA_TITLE, getString(R.string.combination) + " 1/2");
    	startActivityForResult(intent, ITEM_COMBINATION_1);
	}

	/**
	 * Perform a combination of multiple processing.
	 * 2/2: Increases the value of the count read,
	 * and writes back and shows the count to the display.
	 */
	private void execCombination2(Bundle bundleFrom){
		byte[] readData = bundleFrom.getByteArray(MpHelper.EXTRA_USER_DATA);
		Tag tag = bundleFrom.getParcelable(MpHelper.EXTRA_ACTIVE_TAG);
		byte[] idm = bundleFrom.getByteArray(MpHelper.EXTRA_IDM);

		int countRead = readData[0] + 1;
		mOutputView.setText(String.format("Count = %d", countRead));

		byte[] writeData = new byte[]{(byte)countRead};

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_WRITE_SHOW);
		//write
		intent.putExtra(MpHelper.EXTRA_START_ADDRESS, 0);
		intent.putExtra(MpHelper.EXTRA_USER_DATA, writeData);

		//image
		String text = String.format("%d", countRead);
		intent.putExtra(MpHelper.EXTRA_BITMAP, createTextImage(text, 80));
		intent.putExtra(MpHelper.EXTRA_DITHER, true);

		//These parameters are required only in NFC. -->
		Bundle bundle = new Bundle();
		bundle.putBoolean(MpHelper.EXTRA_RELAY_TAG, true);
		bundle.putParcelable(MpHelper.EXTRA_ACTIVE_TAG, tag);
		bundle.putByteArray(MpHelper.EXTRA_IDM, idm);
		intent.putExtras(bundle);
		//<--

		//Optional
		intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);
		intent.putExtra(MpHelper.EXTRA_TITLE, getString(R.string.combination) + " 2/2");
		intent.putExtra(MpHelper.EXTRA_VIBRATE, MpHelper.VIBRATE_ON);
		intent.putExtra(MpHelper.EXTRA_VIBRATE_DURATION, 200);

    	startActivityForResult(intent, ITEM_COMBINATION_2);
	}

	/**
  	 * Convert a byte array to hex string.
  	 */
	public static String makeHexText(byte[] data){

		StringBuffer sb = new StringBuffer();
    	for(int i = 0; i < data.length; i++){
    		if(i != 0){
    			sb.append("-");
    		}
    		sb.append(String.format("%02X", data[i]));
    	}
    	return sb.toString();
    }

	/**
	 * Change a security code.
	 */
	private void changeSecurityCode(){
		/*
		 * CAUTION!
		 * If you forget new security code,
		 * you will not be able to access the feature.
		 */
		byte[] oldCode1 = new byte[]{0x30, 0x30, 0x30};
		byte[] newCode1 = new byte[]{0x30, 0x30, 0x30}; //TODO:Change to any code.

		byte[] oldCode3 = new byte[]{0x30, 0x30, 0x30};
		byte[] newCode3 = new byte[]{0x30, 0x30, 0x30}; //TODO:Change to any code.

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		//Required
		intent.putExtra(MpHelper.EXTRA_SMART_TAG_TYPE, MpHelper.SMART_TAG_TYPE_27);
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_CHANGE_SECURITY_CODE);

		//Security code 1 (for display control)
		intent.putExtra(MpHelper.EXTRA_SECURITY_CODE1, oldCode1);
		intent.putExtra(MpHelper.EXTRA_NEW_SECURITY_CODE1, newCode1);

		//Security code 2 (for writing memory)
		//Not specified -> Not changed

		//Security Code 3 (for reading memory)
		intent.putExtra(MpHelper.EXTRA_SECURITY_CODE3, oldCode3);
		intent.putExtra(MpHelper.EXTRA_NEW_SECURITY_CODE3, newCode3);

		intent.putExtra(MpHelper.EXTRA_CHECK_STATUS_AFTER_PROCESS, true);

    	startActivityForResult(intent, ITEM_CHANGE_SECURITY_CODE);
	}

	/**
	 * Shows the result of changing security-code.
	 * @param bundle
	 */
	private void checkSecurityChangeResult(Bundle bundle){
		byte status = (byte)bundle.getByte(MpHelper.EXTRA_TAG_STATUS, (byte)0);
		String message;
		if(status == (byte)0xf0){
			message = getString(R.string.securityCodeChanged);
		}else if(status == (byte)0xf9){
			message = "Error: " + getString(R.string.securityCodeMismatched);
		}else{
			message = String.format("Error: status = 0x%02X", status);
		}
		mOutputView.setText(message);
	}

	/**
	 * Executes a multiple function.
	 * (reads data -> writes data -> shows an image)
	 */
	private void executeMultiFunction(){
		int testSize = 1024;
		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);
		intent.putExtra(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_READ_WRITE_SHOW);

		//read
		intent.putExtra(MpHelper.EXTRA_START_ADDRESS, 0);
    	intent.putExtra(MpHelper.EXTRA_READ_SIZE, testSize);

		//write
		byte[] writeData = new byte[testSize];
		for(int i = 0; i < testSize; i++){
			writeData[i] = (byte)0x30;
		}
		intent.putExtra(MpHelper.EXTRA_START_ADDRESS2, 0);
		intent.putExtra(MpHelper.EXTRA_USER_DATA, writeData);

		//show image
		intent.putExtra(MpHelper.EXTRA_BITMAP, mBitmap);
		intent.putExtra(MpHelper.EXTRA_DITHER, true);

		//Optional
    	intent.putExtra(MpHelper.EXTRA_SHOW_PROGRESS, true);

		startActivityForResult(intent, ITEM_MULTI_FUNCTION);
	}

	/**
	 * Creates a text image.
	 * @param text
	 * @param size
	 * @return
	 */
    private static Bitmap createTextImage(String text, int size){
    	final float padding = 0;
    	final int width = 200;
    	final int height = 96;

    	Bitmap bmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
    	Canvas canvas = new Canvas(bmp);
    	canvas.drawColor(Color.WHITE);

    	Paint paint = new Paint();
    	paint.setAntiAlias(false);
    	paint.setColor(Color.BLACK);
    	paint.setTextSize(size);
    	FontMetrics metrics = paint.getFontMetrics();

    	float x = padding;
    	float y = padding - metrics.ascent;
    	canvas.drawText(text, x, y, paint);

    	return bmp;
    }

}