/*
 * Copyright (c) 2015, AIOI・SYSTEMS CO., LTD.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */
package com.aioisystems.marbleportsample;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.NfcF;
import android.os.Bundle;
import android.util.Log;

public class RelayActivity extends Activity {
	private static final String LOG_TAG = "MyApp";
	private static final int INIT_NFC_OK = 0;
	private static final int INIT_NFC_NOTFOUND = 1;
	private static final int INIT_NFC_DISABLED = 2;
	private static final int INIT_NFC_ERROR = 9;

	private NfcAdapter mAdapter = null;
	private PendingIntent mPendingIntent;
	private IntentFilter[] mFilters;
	private String[][] mTechLists;
	private int mNfcStatus;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.relay);

        mNfcStatus = startNfc();

	}

	@Override
    protected void onResume() {
    	super.onResume();

    	if(mNfcStatus == INIT_NFC_OK){
    		mAdapter.enableForegroundDispatch(
    				this, mPendingIntent, mFilters, mTechLists);
    	}
    }

    @Override
    protected void onPause() {
    	super.onPause();

		if(mNfcStatus == INIT_NFC_OK){
			mAdapter.disableForegroundDispatch(this);
		}
    }

    @Override
	protected void onNewIntent(Intent intent){
    	Tag tag = (Tag)intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
    	byte[] idm = intent.getByteArrayExtra(NfcAdapter.EXTRA_ID);
    	showText(tag, idm);
    }

	/**
     * Prepare for NFC
     * @return 0:Success 1:Not supported 2:Disabled
     */
    private int startNfc(){
        try{
        	mAdapter = NfcAdapter.getDefaultAdapter(this);

        	if(mAdapter == null){
	        	return INIT_NFC_NOTFOUND;
	        }else{
		        if(!mAdapter.isEnabled()){
		        	return INIT_NFC_DISABLED;
		        }
	        	mPendingIntent = PendingIntent.getActivity(this, 0,
		        		new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);

		        mFilters = new IntentFilter[]{
		        		new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED),
		        	};
		        mTechLists = new String[][]{new String[] { NfcF.class.getName() }};
	        }
        }catch(Exception e){
        	Log.e(LOG_TAG, e.toString());
        	return INIT_NFC_ERROR;
        }
        return INIT_NFC_OK;
    }

    /**
	 * Show a text to the smart tag display.
	 */
	private void showText(Tag tag, byte[] idm){
		String text;
		text = "OK!";

		Intent intent = new Intent(MpHelper.MARBLE_PORT_ACTION);

		Bundle bundle = new Bundle();
		//Required
		bundle.putInt(MpHelper.EXTRA_FUNCTION_NO, MpHelper.FN_SHOW_TEXT);
		bundle.putString(MpHelper.EXTRA_DISPLAY_TEXT, text);
		bundle.putInt(MpHelper.EXTRA_TEXT_SIZE, 80);

		bundle.putBoolean(MpHelper.EXTRA_RELAY_TAG, true);
		bundle.putParcelable(MpHelper.EXTRA_ACTIVE_TAG, tag);
		bundle.putByteArray(MpHelper.EXTRA_IDM, idm);

		//Optional
		bundle.putBoolean(MpHelper.EXTRA_SHOW_PROGRESS, true);

		intent.putExtras(bundle);

    	startActivity(intent);
	}

}
